from __future__ import absolute_import

import os
import re


REGEX = (r'# Raincoat: package "(?P<package>[^=]+)==(?P<version>[^"]+)" '
         'path "(?P<path>[^"]+)" (?:"(?P<code_object>[^"]+)")?')


class Match(object):
    def __init__(self, package, version, path,
                 filename, lineno, code_object=None):

        self.package = package
        self.version = version
        self.path = path
        self.filename = filename
        self.lineno = lineno
        self.code_object = code_object

    def __str__(self):
        return (
            "{match.package} == {match.version} "
            "@ {match.path}:{match.code_object} "
            "(from {match.filename}:{match.lineno})".format(match=self))


def find_in_string(file_content, filename):
    for match in re.finditer(REGEX, file_content):
        lineno = lineno = file_content.count(
            os.linesep, 0, match.start()) + 1

        kwargs = match.groupdict()
        yield Match(lineno=lineno, filename=filename, **kwargs)


def find_in_file(filename):
    with open(filename) as handler:
        return find_in_string(handler.read(), filename)


def list_python_files(base_dir="."):
    for root, __, files in os.walk(base_dir):
        for file in files:
            if file.endswith(".py"):
                yield os.path.normpath(os.path.join(root, file))


def find_in_dir(base_dir="."):
    for python_file in list_python_files(base_dir):
        for match in find_in_file(python_file):
            yield match



from __future__ import absolute_import

import ast
from collections import deque

from raincoat.ast_utils import mark_text_ranges


class CodeLocator(ast.NodeVisitor):
    def __init__(self, source, filters):
        self.path = deque()
        self.source = source
        self.filters = filters

    def load(self):
        self.nodes = {}
        nodes = ast.parse(self.source)
        mark_text_ranges(nodes, self.source)
        self.visit(nodes)
        return self.nodes

    def visit_node(self, node):
        self.path.append(node.name)
        node_name = ".".join(self.path)
        if node_name in self.filters:
            self.nodes[node_name] = node
        ast.NodeVisitor.generic_visit(self, node)
        self.path.pop()

    def visit_FunctionDef(self, node):
        self.visit_node(node)

    def visit_ClassDef(self, node):
        self.visit_node(node)


def find_objects(source, code_objects):

    source_lines = source.splitlines()

    if not all(code_objects):
        yield None, source_lines

    if any(code_objects):

        locator = CodeLocator(source=source, filters=code_objects)

        for node_name, node in locator.load().items():
            if node_name in code_objects:
                yield node_name, source_lines[node.lineno - 1:node.end_lineno]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = '0.1.1'

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='raincoat',
    version=version,
    description="Raincoat has your code covered when you can't stay DRY.",
    long_description=readme + '\n\n' + history,
    author='Joachim Jablon',
    author_email='joachim.jablon@people-doc.com',
    url='https://github.com/novafloss/raincoat',
    packages=[
        'raincoat',
    ],
    include_package_data=True,
    install_requires=["sh", "requests", "pip"],
    tests_require=["tox"],
    license="MIT",
    zip_safe=False,
    entry_points={
        'console_scripts': ['raincoat=raincoat:main'],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
